// kollektiv.cc
//
// Beispiel-Programm fuer kollektive Kommunikation

#include <cstdlib>
#include <fstream>
#include <vector>
#include "mpi.h"

using namespace std;

int main(void) {

  const int count=8;
  vector<int> rand_nums(count), max(count), min(count), sum(count), 
    all_rand_nums;

  MPI::Init();  // initialisiere MPI

  int rank=MPI::COMM_WORLD.Get_rank();  // eigenen Rang und 
  int size=MPI::COMM_WORLD.Get_size();  // Zahl der Prozesse ermitteln

  srand(7*rank);  // Zufallszahlengenerator initialisieren
  for (int i=0; i<count; ++i)  // Wuerfeln
    rand_nums[i]=rand()%1000;

  if (rank==0)  // bei empfangendem Prozess Platz reservieren
    all_rand_nums.resize(count*size);
  // Daten einsammeln
  MPI::COMM_WORLD.Gather(&rand_nums[0], count, MPI::INT,
			 &all_rand_nums[0], count, MPI::INT, 0);

  // Maximum, Minimum und Summe berechnen und an Rang 0 senden
  MPI::COMM_WORLD.Reduce(&rand_nums[0], &max[0], count, MPI::INT, MPI::MAX, 0);
  MPI::COMM_WORLD.Reduce(&rand_nums[0], &min[0], count, MPI::INT, MPI::MIN, 0);
  MPI::COMM_WORLD.Reduce(&rand_nums[0], &sum[0], count, MPI::INT, MPI::SUM, 0);

  if (rank==0) { // Ergebnisse speichern
    ofstream out("kollektiv.out");
    for (int i=0; i<size; ++i) {
      out << "Prozess " << i << " :";
      for (int j=0; j<count; ++j)
	out << "\t" << all_rand_nums[i*count+j];
      out << endl;
    }
    out << endl << "Maxima :";
    for (int i=0; i<count; ++i) out << "\t" << max[i];
    out << endl << "Minima :";
    for (int i=0; i<count; ++i) out << "\t" << min[i];
    out << endl << "Summen :";
    for (int i=0; i<count; ++i) out << "\t" << sum[i];
    out << endl;
  }

  MPI::Finalize();   // beende MPI

  return EXIT_SUCCESS;
}
